from django.conf import settings
from django.contrib.auth.models import Group, Permission
from django.core.management.base import BaseCommand
from django.db.models import Q, QuerySet

PERMISSIONS = {
    # Program Managers get all permissions, they also need the edit the permissions
    settings.PRIVILEGED_GROUP: {
        "all program": Permission.objects.filter(content_type__app_label="program"),
        "change user": Permission.objects.filter(codename="change_user"),
    },
    # Host
    settings.ENTITLED_GROUPS[0]: {
        "default add/change episode, episode link, delete episode": Permission.objects.filter(
            codename__in=[
                "add_episode",
                "add_episodelink",
                "change_episode",
                "change_episodelink",
                "delete_episode",
            ],
        ),
        "default change profile": Permission.objects.filter(codename="change_profile"),
        "default add/change/delete media": Permission.objects.filter(
            codename__in=[
                "add_media",
                "change_media",
                "delete_media",
            ]
        ),
        "default add/change/delete media-source": Permission.objects.filter(
            codename__in=[
                "add_mediasource",
                "change_mediasource",
                "delete_mediasource",
            ]
        ),
        "custom add media-source": Permission.objects.filter(
            codename__in=[
                "add__file",
                "add__import",
            ]
        ),
        "custom edit episode": Permission.objects.filter(
            ~Q(codename="edit__episode__topics"),
            ~Q(codename="edit__episode__languages"),
            codename__startswith="edit__episode",
        ),
        "custom edit profile": Permission.objects.filter(codename="edit__profile__name"),
        "custom edit timeslot": Permission.objects.filter(
            codename="edit__timeslot__episode",
        ),
    },
    # Host+
    settings.ENTITLED_GROUPS[1]: {
        "default add/change episode & episode link, delete episode": Permission.objects.filter(
            codename__in=[
                "add_episode",
                "add_episodelink",
                "change_episode",
                "change_episodelink",
                "delete_episode",
            ],
        ),
        "default change profile, schedule, show & timeslot": Permission.objects.filter(
            codename__in=[
                "change_profile",
                "change_schedule",
                "change_show",
                "change_timeslot",
            ],
        ),
        "default add/change/delete media": Permission.objects.filter(
            codename__in=[
                "add_media",
                "change_media",
                "delete_media",
            ]
        ),
        "default add/change/delete media-source": Permission.objects.filter(
            codename__in=[
                "add_mediasource",
                "change_mediasource",
                "delete_mediasource",
            ]
        ),
        "custom add media-source": Permission.objects.filter(
            codename__in=[
                "add__file",
                "add__import",
                "add__line",
                "add__stream",
            ]
        ),
        "custom edit episode": Permission.objects.filter(
            ~Q(codename="edit__episode__topics"),
            codename__startswith="edit__episode",
        ),
        "custom edit profile": Permission.objects.filter(
            codename__in=[
                "edit__profile__biography",
                "edit__profile__email",
                "edit__profile__image",
                "edit__profile__links",
                "edit__profile__name",
            ]
        ),
        "custom edit schedule": Permission.objects.filter(
            codename="edit__schedule__default_media_id"
        ),
        "custom edit show": Permission.objects.filter(
            codename__in=[
                "edit__show__default_media_id",
                "edit__show__description",
                "edit__show__email",
                "edit__show__hosts",
                "edit__show__image",
                "edit__show__links",
                "edit__show__logo",
                "edit__show__short_description",
            ]
        ),
        "custom edit timeslot": Permission.objects.filter(
            codename="edit__timeslot__episode",
        ),
    },
}


class Command(BaseCommand):
    help = "add permissions"

    def add_permissions(self, group: Group, permissions: QuerySet, name: str):
        self.stdout.write(f"Adding {name} permissions to the '{group.name}' group... ", ending="")
        self.stdout.flush()
        group.permissions.add(*permissions)
        self.stdout.write(self.style.SUCCESS(str(len(permissions))))

    def handle(self, *args, **options):
        for group_name in PERMISSIONS:
            group = Group.objects.get(name=group_name)

            for name, permissions in PERMISSIONS[group_name].items():
                self.add_permissions(group, permissions, name)
